package user

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/uuid"
	"github.com/pkg/errors"
	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
	"live-service/app/user_center/model"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"
)

type RetrievePlatformUserLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext

	logx.Logger
}

func NewRetrievePlatformUserLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RetrievePlatformUserLogic {
	return &RetrievePlatformUserLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// RetrievePlatformUser 查询或创建用户（此时不通过API查询平台用户信息）
func (l *RetrievePlatformUserLogic) RetrievePlatformUser(in *pb.PlatformUserReq) (*pb.PlatformUserResp, error) {
	var displayUser *model.UserForDisplay

	if err := l.svcCtx.Db.Transaction(func(tx *gorm.DB) error {
		var err error
		if displayUser, err = l.svcCtx.UserPlatformModel.FindDisplayOneByPlatform(l.ctx, tx, in.Platform, in.PUid); err != nil {
			if !errors.Is(err, model.ErrNotFound) {
				return nerr.NewWithCode(nerr.DBError)
			}
		} else {
			return nil
		}
		// : not found

		// insert
		newId := uuid.NextId()
		if err := l.svcCtx.UserModel.Insert(l.ctx, tx, &model.User{Id: newId}); err != nil {
			return nerr.NewWithCode(nerr.DBError)
		}
		dbPlatformUser := &model.UserPlatform{
			Id:       uuid.NextId(),
			UserId:   newId,
			Platform: in.Platform,
			PUid:     in.PUid,
			PInfo:    "{}",
		}
		if err := l.svcCtx.UserPlatformModel.Insert(l.ctx, tx, dbPlatformUser); err != nil {
			return nerr.NewWithCode(nerr.DBError)
		}
		displayUser = &model.UserForDisplay{
			UserId: newId,
		}
		return nil
	}); err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "查询或创建用户-事务执行失败, err:%+v", err)
	}
	return &pb.PlatformUserResp{
		Id:       displayUser.UserId,
		Platform: in.Platform,
		PUid:     in.PUid,
		PUname:   displayUser.PUname,
		PAvatar:  displayUser.PAvatar,
	}, nil
}
