package zhg

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/gorm-zero/gormx"
	"github.com/pkg/errors"
	"live-service/app/user_center/model"
	"live-service/common/nerr"
	"live-service/common/timex"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type ChangeEliteLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewChangeEliteLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ChangeEliteLogic {
	return &ChangeEliteLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *ChangeEliteLogic) ChangeElite(in *pb.EliteReq) (*pb.ChangeEliteResp, error) {
	resp := &pb.ChangeEliteResp{
		UserId: in.UserId,
	}
	eliteId := l.svcCtx.GameConfig.Elite.Default.Id
	if in.Sort != l.svcCtx.GameConfig.Elite.Default.Sort {
		dbModel, err := l.svcCtx.UserEliteModel.FindOneByUserIdSort(l.ctx, nil, in.UserId, int64(in.Sort))
		if err != nil {
			if errors.Is(err, gormx.ErrNotFound) {
				return nil, errors.Wrap(nerr.NewError(nerr.UserEliteNotFoundErr, "用户没有此单位"), "用户没有此单位")
			}
			return nil, errors.Wrap(nerr.NewWithCode(nerr.DBError), "数据库错误，请联系UP主")
		}
		// 判断过期
		if timex.DayExpire(time.Now(), dbModel.EndTime, bool(dbModel.Forever)) {
			return nil, errors.Wrap(nerr.NewError(nerr.UserEliteExpiresErr, "待切换的精英单位已过期"), "待切换的精英单位已过期")
		}
		eliteId = dbModel.EliteId
	}

	// 更新当前使用精英单位
	err := l.svcCtx.ZhgUserDetailsModel.Upsert(l.ctx, nil, &model.UpsertParam{
		UserId: in.UserId,
		Elite:  &eliteId,
	})
	if err != nil {
		if !errors.Is(err, gormx.ErrRowsAffectedZero) {
			return nil, errors.Wrapf(nerr.NewError(nerr.UserChangeEliteErr, "切换精英单位失败"), "切换精英单位失败 err:%v", err)
		}
	}

	resp.EliteId = eliteId

	return resp, nil
}
